@@ -11,6 +11,7 @@ from typing import Dict, List, Optional, Type
 import urllib.parse
 
 from aiohttp import ClientSession, ClientTimeout
+from aiohttp.connector import TCPConnector
 import humps
 
 from . import __version__
@@ -20,6 +21,7 @@ from .const import (
     DEFAULT_USER_AGENT,
     XMO_ACCESS_RESTRICTION_ERR,
     XMO_AUTHENTICATION_ERR,
+    XMO_MAX_SESSION_COUNT_ERR,
     XMO_NO_ERR,
     XMO_NON_WRITABLE_PARAMETER_ERR,
     XMO_REQUEST_ACTION_ERR,
@@ -32,6 +34,7 @@ from .exceptions import (
     AuthenticationException,
     BadRequestException,
     LoginTimeoutException,
+    MaximumSessionCountException,
     NonWritableParameterException,
     UnauthorizedException,
     UnknownException,
@@ -50,6 +53,8 @@ class SagemcomClient:
         password,
         authentication_method,
         session: ClientSession = None,
+        ssl=False,
+        verify_ssl=True,
     ):
         """
         Create a SagemCom client.
@@ -65,6 +70,8 @@ class SagemcomClient:
         self.authentication_method = authentication_method
         self._password_hash = self.__generate_hash(password)
 
+        self.protocol = "https" if ssl else "http"
+
         self._current_nonce = None
         self._server_nonce = ""
         self._session_id = 0
@@ -76,6 +83,7 @@ class SagemcomClient:
             else ClientSession(
                 headers={"User-Agent": f"{DEFAULT_USER_AGENT}/{__version__}"},
                 timeout=ClientTimeout(DEFAULT_TIMEOUT),
+                connector=TCPConnector(ssl=verify_ssl),
             )
         )
 
@@ -164,15 +172,11 @@ class SagemcomClient:
 
     async def __api_request_async(self, actions, priority=False):
         """Build request to the internal JSON-req API."""
-        # Auto login
-        if self._server_nonce == "" and actions[0]["method"] != "logIn":
-            await self.login()
-
         self.__generate_request_id()
         self.__generate_nonce()
         self.__generate_auth_key()
 
-        api_host = f"http://{self.host}{API_ENDPOINT}"
+        api_host = f"{self.protocol}://{self.host}{API_ENDPOINT}"
 
         payload = {
             "request": {
@@ -232,6 +236,9 @@ class SagemcomClient:
                         if action_error_description == XMO_UNKNOWN_PATH_ERR:
                             raise UnknownPathException(action_error)
 
+                        if action_error_description == XMO_MAX_SESSION_COUNT_ERR:
+                            raise MaximumSessionCountException(action_error)
+
                         raise UnknownException(action_error)
 
                 return result
@@ -277,6 +284,16 @@ class SagemcomClient:
         else:
             raise UnauthorizedException(data)
 
+    async def logout(self):
+        """Log out of the Sagemcom F@st device."""
+        actions = {"id": 0, "method": "logOut"}
+
+        await self.__api_request_async([actions], False)
+
+        self._session_id = -1
+        self._server_nonce = ""
+        self._request_id = -1
+
     async def get_value_by_xpath(
         self, xpath: str, options: Optional[Dict] = {}
     ) -> Dict:
@@ -317,15 +334,36 @@ class SagemcomClient:
         }
 
         response = await self.__api_request_async([actions], False)
-        print(response)
 
         return response
 
     async def get_device_info(self) -> DeviceInfo:
         """Retrieve information about Sagemcom F@st device."""
-        data = await self.get_value_by_xpath("Device/DeviceInfo")
-
-        return DeviceInfo(**data.get("device_info"))
+        try:
+            data = await self.get_value_by_xpath("Device/DeviceInfo")
+            return DeviceInfo(**data.get("device_info"))
+        except UnknownPathException:
+            device_info = DeviceInfo()
+            device_info.mac_address = await self.get_value_by_xpath(
+                "Device/DeviceInfo/MACAddress"
+            )
+            device_info.manufacturer = "Sagemcom"
+            device_info.model_name = await self.get_value_by_xpath(
+                "Device/DeviceInfo/ModelNumber"
+            )
+            device_info.model_number = await self.get_value_by_xpath(
+                "Device/DeviceInfo/ProductClass"
+            )
+            device_info.product_class = await self.get_value_by_xpath(
+                "Device/DeviceInfo/ProductClass"
+            )
+            device_info.serial_number = await self.get_value_by_xpath(
+                "Device/DeviceInfo/SerialNumber"
+            )
+            device_info.software_version = await self.get_value_by_xpath(
+                "Device/DeviceInfo/SoftwareVersion"
+            )
+            return device_info
 
     async def get_hosts(self, only_active: Optional[bool] = False) -> List[Device]:
         """Retrieve hosts connected to Sagemcom F@st device."""
